function varargout=configure(varargin)
% FUNCTION fc_oogmsh.configure()
%   Configures the toolbox/package by setting gmsh path, meshes default 
%   directory and .geo files default directory.
%   Theses informations will be stored in ... file.
%
% <COPYRIGHT>
  [conffile,isFileExists]=fc_oogmsh.getLocalConfFile();
  if isFileExists
    run(conffile);
  else
    fullname=mfilename('fullpath');
    I=strfind(fullname,filesep);
    Path=fullname(1:(I(end-1)-1));
    gmsh_bin=getDefaultGmshBin();
    mesh_dir=[Path,filesep,'meshes'];
    geo_dir=[Path,filesep,'geodir'];
    fc_tools_dir=''; % empty if pkg or toolbox in path
  end
  p = inputParser;
  p.addParamValue('verbose', 1, @(x) ismember(x,0:2) ); % level of verbosity 
  p.addParamValue('gmsh_bin',gmsh_bin,@ischar);
  p.addParamValue('mesh_dir',mesh_dir,@ischar);
  p.addParamValue('geo_dir',geo_dir,@ischar);
  p.addParamValue('fc_tools_dir',fc_tools_dir,@ischar);
  %p.addParamValue('force',false,@islogical); 
  p.parse(varargin{:});
  R=p.Results;verbose=R.verbose;
  if length(R.gmsh_bin)==0,R.gmsh_bin=getDefaultGmshBin();end
  
  if ~isdir(R.mesh_dir)
    try
      mkdir(R.mesh_dir);
    catch
      error('[fc-oogmsh.configure] Unable to create directory: %s\n',R.mesh_dir)
    end
  end
  
  %if ~isdir(R.mesh_dir), error('Not a directory :\n''mesh_dir''=%s\n', R.mesh_dir); end
  if ~isdir(R.geo_dir), error('[fc-oogmsh.configure] Not a directory :\n''geo_dir''=%s\n', R.geo_dir); end
  %if exist(R.gmsh_bin)~=2, error('Not a file :\n''gmsh_bin''=%s\n', R.gmsh_bin); end
  if ismac()
    if exist( R.gmsh_bin)~=2
      R.gmsh_bin=fullfile(R.gmsh_bin,'Contents','MacOS','gmsh');
    end
  end
  if exist( R.gmsh_bin)~=2
    fprintf('[fc-oogmsh]  -> Select gmsh binary ...\n');
    if ispc()
      [gmsh_bin, gmsh_dir]=uigetfile('*.exe','Select GMSH binary',R.gmsh_bin);
    else
      [gmsh_bin, gmsh_dir]=uigetfile('*','Select GMSH binary',R.gmsh_bin);
      if ismac() && strcmp(gmsh_bin,'Gmsh.app')
        gmsh_dir=fullfile(gmsh_dir,gmsh_bin,'Contents','MacOS');
        gmsh_bin='gmsh';
      end
    end
    R.gmsh_bin=fullfile(gmsh_dir,gmsh_bin);
    if exist(R.gmsh_bin)~=2
      warning('[fc-oogmsh.configure] Failed to load GMSH binary \n  -> %s\n',R.gmsh_bin)
    end
  %else
  %  [gmsh_dir,gmsh_bin,gmsh_ext]=fileparts(R.gmsh_bin);
  %  if ~isempty(gmsh_ext),gmsh_bin=[gmsh_bin,'.',gmsh_ext];end
  end
  vprintf(verbose,2,'[fc-oogmsh] Using GMSH binary : %s\n',R.gmsh_bin);
  
  fc_tools_dir=get_tbxpath('tools','oogmsh',R.fc_tools_dir);
  
  fprintf('[fc-oogmsh] Writing in %s ...\n',conffile);
  fid=fopen(conffile,'w');
  if (fid==0), error('[fc-oogmsh.configure] Unable to open file :\n %s\n',conffile);end
  fprintf(fid,'%% Automaticaly generated with fc_oogmsh.configure()\n');
  fprintf(fid,'gmsh_bin=''%s'';\n',R.gmsh_bin);
  fprintf(fid,'mesh_dir=''%s'';\n',R.mesh_dir);
  fprintf(fid,'geo_dir=''%s'';\n',R.geo_dir);
  fprintf(fid,'fc_tools_dir=''%s'';\n',fc_tools_dir);
  fclose(fid);
  vprintf(verbose,2,'[fc-oogmsh] configured with\n');
  vprintf(verbose,2,'   ->     gmsh_bin=''%s'';\n',R.gmsh_bin);
  vprintf(verbose,2,'   ->     mesh_dir=''%s'';\n',R.mesh_dir);
  vprintf(verbose,2,'   ->      geo_dir=''%s'';\n',R.geo_dir);
  vprintf(verbose,2,'   -> fc_tools_dir=''%s'';\n',fc_tools_dir);
  vprintf(verbose,2,'[fc-oogmsh] done\n');
  %eval(sprintf('clear %s', conffile))
  if nargout==1,varargout{1}=conffile;end
  rehash
end

function default_gmsh_bin=getDefaultGmshBin()
  if ispc()
    default_gmsh_bin=fullfile([getenv('HOMEDRIVE') getenv('HOMEPATH')],'Softwares','GMSH','gmsh.exe');
  elseif ismac()
    default_gmsh_bin=fullfile(getenv('HOME'),'GMSH','Gmsh.app','Contents','MacOS','gmsh');
  else
    default_gmsh_bin=fullfile(getenv('HOME'),'bin','gmsh');
  end
end

%
% DO NOT MODIFY THIS FUNCTION: IT IS AUTOMATICALLY ADDED
%
function tbxpath=get_tbxpath(tbxname,tbxfrom,givenpath,varargin) % tbxname is the toolbox to 'include' in the toolbox <tbxfrom>
  p = inputParser;
  p.addParamValue('stop'   ,true,@islogical);
  p.addParamValue('verbose', 1, @(x) ismember(x,0:2) ); % level of verbosity 
  p.parse(varargin{:});
  stop=p.Results.stop;verbose=p.Results.verbose;
  tbxpath='';
  if ~isempty(givenpath) 
    if ~isdir(givenpath) 
      vprintf(verbose,2,'[fc-%s] The given path does not exists:\n   -> %s\n',tbxfrom,tbxname,givenpath)
      vprintf(verbose,2,'[fc-%s] Use fc_%s.configure(''fc_%s_dir'',<DIR>) to correct this issue\n\n',tbxfrom,tbxfrom,tbxname)
      if stop
        error(sprintf('fc-%s::configure',tbxfrom))
      else
        warning(sprintf('fc-%s::configure',tbxfrom))
      end
    end
    tbxpath=givenpath;
    addpath(tbxpath);rehash path;
    failed=false;
    try % check if the toolbox can be found
      eval(sprintf('fc_%s.version();',tbxname))
    catch
      vprintf(verbose,2,'[fc-%s] Unable to load the fc-%s toolbox/package in given path:\n  %s\n',tbxfrom,tbxname,tbxpath)
      if stop
        error(sprintf('fc-%s::configure',tbxfrom),'step 1')
%        else
%          warning(sprintf('fc-%s::configure',tbxfrom),'step 1')
      end
      failed=true;
    end
    rmpath(tbxpath);rehash path;
    if ~failed, return;end
  end
  failed=false;
  try % check if the toolbox is in current Matlab path
    eval(sprintf('fc_%s.version();',tbxname))
  catch
    vprintf(verbose,2,'[fc-%s] Unable to load the fc-%s toolbox/package in current path\n',tbxfrom,tbxname)
    failed=true;
  end
  if ~failed, return;end
  failed=false;
  fullname=mfilename('fullpath');
  I=strfind(fullname,filesep);
  Path=fullname(1:(I(end-2)-1));
  
  lstdir=dir(Path); % try to guess directory. I don't use dir command due to trouble with octave
  C=arrayfun(@(x) x.name,lstdir, 'UniformOutput', false);
  I=strfind(C,['fc-',tbxname]);
  i=find(cellfun(@(x) ~isempty(x),I)==1);
  if ~isempty(i)
    k=1;
    while k<=length(i)
      tbxpath=[Path,filesep,C{i(k)}];
      addpath(tbxpath);rehash path;
      try % check if the toolbox is in new current Matlab path
        eval(sprintf('fc_%s.version();',tbxname))
      catch
        failed=true;
        vprintf(verbose,2,'[fc-%s] Unable to load the fc-%s toolbox/package in guess path\n  %s\n',tbxfrom,tbxname,tbxpath)
        vprintf(verbose,2,'[fc-%s] Use fc_%s.configure(''fc_%s_dir'',<DIR>) to correct this issue\n\n',tbxfrom,tbxfrom,tbxname)
        if stop
          error(sprintf('fc-%s::configure',tbxfrom),'step 2')
%          else
%            warning(sprintf('fc-%s::configure',tbxfrom),'step 2')
        end
      end
      rmpath(tbxpath);rehash path;
      if ~failed
        vprintf(verbose,2,'[fc-%s] Loading the fc-%s toolbox/package in guess path\n  %s\n',tbxfrom,tbxname,tbxpath)
        return;
      end
      k=k+1;
    end
  else
    vprintf(verbose,2,'[fc-%s] Guess path does not exists:\n   -> %s\n',tbxfrom,tbxname,tbxpath)
    vprintf(verbose,2,'[fc-%s] Use fc_%s.configure(''fc_%s_dir'',<DIR>) to correct this issue\n\n',tbxfrom,tbxfrom,tbxname);
    if stop
      error(sprintf('fc-%s::configure',tbxfrom),'step 3')
%      else
%        warning(sprintf('fc-%s::configure',tbxfrom),'step 3')
    end
    failed=true;
  end
  if failed
      tbxpath='';
  else
    vprintf(verbose,2,'[fc-%s] fc-%s toolbox/package found in path:\n   -> %s\n',tbxfrom,tbxname,tbxpath)
  end
end

function vprintf(verbose,level,varargin)
  if verbose>=level, fprintf(varargin{:});end
end

